#ifndef BPLCC_GL_2_3_H
#define BPLCC_GL_2_3_H
/* Required Header Files & Libraries */
#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <math.h>
#include <mmsystem.h>
#include <stdio.h>
#pragma lib <opengl32.lib>
#pragma lib <glu32.lib>
#pragma lib <winmm.lib>

/* ON PROGRAMMING USER DEFINITIONS ARE AT THE END OF THIS FILE - THE OTHERS AT START*/
#define true 1
#define false 0
typedef unsigned char bool;

/********************* Required Functions That User/Programmer has to make. ***********************************/
void ToDo_WinMain(void);				// Called at the start of "main" function. (main-->winmain).
void ToDo_Init_OpenGL(void);		// Initialization Function After OpenGL & Window has Loaded.
void ToDo_MainLoop(void);				// MainLoop Function. Called Continuous (f.e. use here variables that change with time).
void ToDo_RenderScene(void);		// Render Scene Function. Use this Function To Render you OpenGL Scene.
void ToDo_AtExitProgram(void);  // At Exit Application Called Function.
/* Use the below function to trap Keyboard / Mouse input! */
void ToDo_WinProcMessage(UINT uMsg, WPARAM wParam, LPARAM lParam);
/* At This Function Make Your Window as you want it with BPLcc_WinData_Set-Functions. */
void BPLcc_OpenGL_Window_Parameters_Set(void);

/*****************************************************************************************************************/
/******************************************** Objects Declerations ***********************************************/
/*****************************************************************************************************************/
                                                                                // BASIC STRUCTURE
/*--------------------------------------------------------------------------------------------------*/
/* The Object That Handles The Parameters for the Window Processes - BASIC*/
struct sct_BPLcc_WinData {
	char Name[256];
	char ClassName[256];
	int Style, BackgroundBrush, FullScreenApplicationStyle, WindowScreenApplicationStyle;
	int PF_Flags, PF_LayerMask, PF_PixelType, PF_AccumBits, PF_StencilBits;
	HWND hWnd;
	RECT rRect;
	HDC hDC;
	HGLRC hRC;
	HINSTANCE hInst, hInstICON, hInstCURSOR;
	LPCSTR DefaultIcon;
	LPCSTR DefaultCursor;

	int ScreenX, ScreenY, FullScreen, ScreenDepth, StartCursorOnOff, ResizedWindow, ExitOnESC;
	int WinPosX, WinPosY;
	int GLFrustumData[6];
};
typedef struct sct_BPLcc_WinData BPLcc_WinData;                                 // POINT3D Struct
/*--------------------------------------------------------------------------------------------------*/
/* Point "class" Decleration */
struct sct_BPLcc_POINT3 {
	float x, y, z;
};
typedef struct sct_BPLcc_POINT3 BPLcc_POINT3;                                  // Camera Struct
/*--------------------------------------------------------------------------------------------------*/
/* Camera "class" Decleration */
struct sct_BPLcc_Camera {
	BPLcc_POINT3 Pos, View, UpVector, Strafe;
	double Parameters[256];
};
typedef struct sct_BPLcc_Camera BPLcc_Camera;                                 // Keyboard Struct
/*--------------------------------------------------------------------------------------------------*/
/* Keyboard "Class" Decleration */
struct sct_BP_KeyBoard {
	bool keys[256], keysb[256];
	int Last;
	char IString[1024];
	bool StringInput;
	int StrNum, MaxChars;
};
typedef struct sct_BP_KeyBoard BPLcc_Keys;                                    // Mouse Struct
/*--------------------------------------------------------------------------------------------------*/
/* Mouse "Class" Decleration */
struct str_BPLcc_Mouse {
	bool keys[7], keysb[7], bCursorState;
	int Last, CursorState;
	short Wheel;
	POINT pos;
};
typedef struct str_BPLcc_Mouse BPLcc_Mouse;                                 // Simple Fonts Struct
/*--------------------------------------------------------------------------------------------------*/
/* Simple Fonts "Class" Decleration */
struct sct_BPLcc_Fonts {
	int ScreenH; HDC* hDC; int max_Chars;
	LPSTR str; int height, weight, orien, escp, width;
	bool ital, under, ObjInitialized;
	UINT FontListID; HFONT hOldFont;
	float Red, Green, Blue, Alpha;
	float FRed, FGreen, FBlue, FAlpha;
	float	Fonts_Raster4f_Z;
};
typedef struct sct_BPLcc_Fonts BPLcc_Font;                                     // 3D Fonts Struct
/*--------------------------------------------------------------------------------------------------*/
/* 3D Fonts "Class" Decleration */
#define BPLCC_MAX_CHARS	256
struct sct_BPLcc_FONT3D {
	UINT FontListID;
	HFONT hOldFont;
	GLYPHMETRICSFLOAT GlyphInfo[BPLCC_MAX_CHARS];
	HDC* hDC;
	int Height, Bold, Width, Escapement, Orientation;
	float Extrude;
	bool Italic, Underline, Strikeout;
	LPSTR FontName;
	float Red, Green, Blue, Alpha;
	float FRed, FGreen, FBlue, FAlpha;
	int Type;
};
typedef struct sct_BPLcc_FONT3D BPLcc_Font3D;                                 // BASIC Image Struct
/*--------------------------------------------------------------------------------------------------*/
// Basic Image Structure.. All image formats must end at this structure in order to use textures.
struct sct_BPLcc_Image{
	unsigned int sizeX, sizeY;
  unsigned char *data;
};
typedef struct sct_BPLcc_Image BPLcc_Image;                                // Texture Handling Struct
/*--------------------------------------------------------------------------------------------------*/
// OpenGL Texture Support Object.
struct sct_BPLcc_Texture {
	int N;
	unsigned int* TextureMap;
};
typedef struct sct_BPLcc_Texture BPLcc_Texture2D;                              // Fog Handler
/*--------------------------------------------------------------------------------------------------*/
// Fog Support Object.
struct sct_BPLcc_Fog {
	int FMode,FHint;
	float FDen, FStart, FEnd;
	float FColor[4];
};
typedef struct sct_BPLcc_Fog BPLcc_Fog;


/*****************************************************************************************************************/
/************************************* Objects Functions Declarations ********************************************/
/*****************************************************************************************************************/

/************************************* WinData Set Functions *****************************************************/
/* Functions That give the capability to change the Parameters of BPLcc_InitWinData_Obj */
void BPLcc_WinData_Set(BPLcc_WinData *hObj, int SelectValue, int SetValue);
void BPLcc_WinData_SetPc(BPLcc_WinData *hObj, int SelectValue, char* SetValue);
void BPLcc_WinData_SetPi(BPLcc_WinData *hObj, int SelectValue, int* SetValue, int N);
void BPLcc_WinData_SetID(BPLcc_WinData *hObj, int SelectValue, LPCSTR SetValue, bool Default);

/************************************* POINT3 Functions *****************************************************/
void BPLcc_POINT3_Create(BPLcc_POINT3 *hObj, float x, float y, float z);
void BPLcc_POINT3_Copy(BPLcc_POINT3 *hObj, BPLcc_POINT3 *hObj2);
BPLcc_POINT3 BPLcc_POINT3_Add(BPLcc_POINT3 *hObj, BPLcc_POINT3 *hObj2);
BPLcc_POINT3 BPLcc_POINT3_Sub(BPLcc_POINT3 *hObj, BPLcc_POINT3 *hObj2);
BPLcc_POINT3 BPLcc_POINT3_Mulf(BPLcc_POINT3 *hObj, float num);
BPLcc_POINT3 BPLcc_POINT3_Divf(BPLcc_POINT3 *hObj, float num);
/* Extra 3D Point Math Functions */
BPLcc_POINT3 BPLcc_POINT3_Cross(BPLcc_POINT3 *hObj, BPLcc_POINT3 *hObj2);
float BPLcc_POINT3_Magnitude(BPLcc_POINT3 *hObj);
BPLcc_POINT3 BPLcc_POINT3_RetNormalize(BPLcc_POINT3 *hObj);
void BPLcc_POINT3_Normalize(BPLcc_POINT3 *hObj);

/************************************* CAMERA Functions *****************************************************/
void BPLcc_CAM_Create(BPLcc_Camera* hObj, int ScreenX, int ScreenY);
void BPLcc_CAM_Place(BPLcc_Camera* hObj,
			float posx, float posy, float posz, float viewx, float viewy, float viewz, float upvx, float upvy, float upvz);
void BPLcc_CAM_PlaceV(BPLcc_Camera* hObj, BPLcc_POINT3 *Pos, BPLcc_POINT3 *View, BPLcc_POINT3 *UpVector);
void BPLcc_CAM_RotateView(BPLcc_Camera* hObj, float angle, float x, float y, float z);
void BPLcc_CAM_SetViewByMouse(BPLcc_Camera* hObj);
void BPLcc_CAM_RotateAroundPoint(BPLcc_Camera* hObj, float cx, float cy, float cz, float angle, float x, float y, float z);
void BPLcc_CAM_RotateAroundPointV(BPLcc_Camera* hObj, BPLcc_POINT3 *Center, float angle, float x, float y, float z);
void BPLcc_CAM_Strafe(BPLcc_Camera* hObj, float Speed);
void BPLcc_CAM_Move(BPLcc_Camera* hObj, float Speed);
void BPLcc_CAM_Update(BPLcc_Camera* hObj);
void BPLcc_CAM_Look(BPLcc_Camera* hObj);
void BPLcc_CAM_Parameter(BPLcc_Camera* hObj, int mode, double ParPass);
void BPLcc_CAM_SetSpeed(BPLcc_Camera* hObj, double Speed);
void BPLcc_CAM_CheckForMovement(BPLcc_Camera* hObj);

/* Extra Camera Functions */
float BPLcc_CAM_CalcFrameRate(void);

/************************************* Keyboard Functions *****************************************************/
void BPLcc_Keys_Create(BPLcc_Keys* hObj);
void BPLcc_Keys_Update(BPLcc_Keys* hObj, UINT uMsg, WPARAM wParam);
bool BPLcc_Keys_Key(BPLcc_Keys* hObj, int key);
void BPLcc_Keys_SetALL(BPLcc_Keys* hObj, bool val);
bool BPLcc_Keys_Shift(BPLcc_Keys* hObj);
bool BPLcc_Keys_Ctrl(BPLcc_Keys* hObj);
void BPLcc_Keys_GetString(BPLcc_Keys* hObj, int maxchars);

/************************************* Mouse Functions *****************************************************/
void BPLcc_Mus_Create(BPLcc_Mouse* hObj);
void BPLcc_Mus_Update(BPLcc_Mouse* hObj, UINT uMsg, WPARAM wParam);
void BPLcc_Mus_ClWheel(BPLcc_Mouse* hObj);
void BPLcc_Mus_ClDBLClk(BPLcc_Mouse* hObj);
bool BPLcc_Mus_Key(BPLcc_Mouse* hObj, int key);
void BPLcc_Mus_SetPos(BPLcc_Mouse* hObj, int x, int y);
void BPLcc_Mus_ShowC(BPLcc_Mouse* hObj, bool state);

/************************************* Simple Font Functions *****************************************************/
void BPLcc_FONTS_Create(BPLcc_Font* hObj);
void BPLcc_FONTS_Destroy(BPLcc_Font* hObj);
void BPLcc_FONTS_SetScreenData(BPLcc_Font* hObj, int scrHeight);
void BPLcc_FONTS_SetMaxChars(BPLcc_Font* hObj, int maxchars);
void BPLcc_FONTS_SetFontsType(BPLcc_Font* hObj, LPSTR str1, int height1, int weight1,
															bool ital1, bool under1, int orien1, int escp1, int width1);
void BPLcc_FONTS_Text(BPLcc_Font* hObj, int x, int y, const char* str, ...);
void BPLcc_FONTS_Color(BPLcc_Font* hObj, float R, float G, float B);
void BPLcc_FONTS_FColor(BPLcc_Font* hObj, float R, float G, float B);
void BPLcc_FONTS_Alpha(BPLcc_Font* hObj, float A, float FA);
void BPLcc_FONTS_PositionText(BPLcc_Font* hObj, int x, int y);

/************************************* 3D Font Functions *****************************************************/
void BPLcc_FONT3D_Create(BPLcc_Font3D* hObj);
void BPLcc_FONT3D_Destroy(BPLcc_Font3D* hObj);
void BPLcc_FONT3D_SetFontType(BPLcc_Font3D* hObj, LPSTR strFontName, int Height, float Extrude,
			bool Italic, int Bold, int Width, bool Underline, bool Strikeout, int Escapement, int Orientation, bool Type);
void BPLcc_FONT3D_Text(BPLcc_Font3D* hObj, float x, float y, float z, const char *strString, ...);
void BPLcc_FONT3D_TextR(BPLcc_Font3D* hObj, float x, float y, float z,  float Rot,
			float Rx, float Ry, float Rz, const char *strString, ...);
void BPLcc_FONT3D_Color(BPLcc_Font3D* hObj, float R, float G, float B);
void BPLcc_FONT3D_FColor(BPLcc_Font3D* hObj, float FR, float FG, float FB);
void BPLcc_FONT3D_Alpha(BPLcc_Font3D* hObj, float A, float FA);
void BPLcc_FONT3D_SetSize(BPLcc_Font3D* hObj, int Size);

/* Version 2.3 */
/************************************* TGA Files Loading Functions *********************************************/
void BPLcc_TGA_Create(BPLcc_Image* hObj, char *Filename);
void BPLcc_TGA_Reload(BPLcc_Image* hObj, char *Filename);
void BPLcc_TGA_Destroy(BPLcc_Image* hObj);
void BPLcc_TGA_Load(BPLcc_Image* hObj, char *Filename);

/************************************* Texture Handling Functions *********************************************/
bool BPLcc_TXRE_Create(BPLcc_Texture2D* hObj, int NumberOfTextures);
bool BPLcc_TXRE_Set(BPLcc_Texture2D* hObj, int Num, BPLcc_Image* Image);
void BPLcc_TXRE_Bind(BPLcc_Texture2D* hObj, int Num);
void BPLcc_TXRE_Destroy(BPLcc_Texture2D* hObj);
void BPLcc_TXRE_TGALoadSet(BPLcc_Texture2D* TXRE_hObj, int Num, char *Filename);

/************************************* Fog Handling Functions *************************************************/
void BPLcc_FOG_Create(BPLcc_Fog* hObj);
void BPLcc_FOG_SetData(BPLcc_Fog* hObj, float density, int mode, float fstart, float fend , int hinttype);
void BPLcc_FOG_Enable(BPLcc_Fog* hObj);
void BPLcc_FOG_Mode(BPLcc_Fog* hObj, int mode);
void BPLcc_FOG_Density(BPLcc_Fog* hObj, float density);
void BPLcc_FOG_Start(BPLcc_Fog* hObj, float fstart);
void BPLcc_FOG_End(BPLcc_Fog* hObj, float fend);
void BPLcc_FOG_Hint(BPLcc_Fog* hObj, int hinttype);
void BPLcc_FOG_Disable(void);
void BPLcc_FOG_Color(BPLcc_Fog* hObj, float r, float g, float b, float a);
void BPLcc_FOG_DenInc(BPLcc_Fog* hObj, float incr);
void BPLcc_FOG_DenDec(BPLcc_Fog* hObj, float decr);


/*****************************************************************************************************************/
/************************************* Definitions For User Use **************************************************/
/*****************************************************************************************************************/
/* Arguments for changing the variables of the parameters of
	BPLcc_InitWinData_Obj, for the Window Processes of the lib... */
#define BPLCC_INITWINOBJ_NAME 1
#define BPLCC_INITWINOBJ_CLASSNAME 2
#define BPLCC_INITWINOBJ_STYLE 3
#define BPLCC_INITWINOBJ_BACKBRUSH 4
#define BPLCC_INITWINOBJ_FULLSCR_APPSTYLE 5
#define BPLCC_INITWINOBJ_WINDOWSCR_APPSTYLE 6
#define BPLCC_INITWINOBJ_PIXF_FLAGS 7
#define BPLCC_INITWINOBJ_PIXF_LAYERMASK 8
#define BPLCC_INITWINOBJ_PIXF_PIXELTYPE 9
#define BPLCC_INITWINOBJ_PIXF_ACCUMBITS 10
#define BPLCC_INITWINOBJ_PIXF_STENCILBITS 11
#define BPLCC_INITWINOBJ_SCR_WIDTH 12
#define BPLCC_INITWINOBJ_SCR_HEIGHT 13
#define BPLCC_INITWINOBJ_SCR_DEPTH 14
#define BPLCC_INITWINOBJ_SCR_FULLSCREEN 15
#define BPLCC_INITWINOBJ_SCR_RESIZEDWINDOW 16
#define BPLCC_INITWINOBJ_SCR_EXITONESCAPE 17
#define BPLCC_INITWINOBJ_SCR_SHOWCURSOR 18
#define BPLCC_INITWINOBJ_GL_FRUSTUMDATA 19
#define BPLCC_INITWINOBJ_ICON 20
#define BPLCC_INITWINOBJ_CURSOR 21
#define BPLCC_INITWINOBJ_WINPOSX 22
#define BPLCC_INITWINOBJ_WINPOSY 23

/*Use BPLcc_CAM_Parameter(&hObj,<Below #defines>, Value) for Camera Parameters Settings*/
#define BPLCC_CAM_SET_PARAMETER_SPEED 0				/* Set Speed Value. (Default: 15) */
#define BPLCC_CAM_SET_PARAMETER_VIEWSPEEDX 3	/* Set X Speed Rotation. The Smaller the Faster (>0) (Default: 800.0f) */
#define BPLCC_CAM_SET_PARAMETER_VIEWSPEEDY 4	/* Set Y Speed Rotation. The Smaller the Faster (>0) (Default: 800.0f) */
#define BPLCC_CAM_SET_PARAMETER_MAXANGLE 5		/* Set The Up/Down Max Angle.. (Default: 1.52f) */
#define BPLCC_CAM_SET_PARAMETER_MOVE 10				/* Set 1/0 to enable/disable default movment of Camera (Default: 1)*/
#define BPLCC_CAM_SET_PARAMETER_VIEW 11				/* Set 1/0 to enable/disable Camera View By Mouse Procedure. (Default: 1) */
#define BPLCC_CAM_SET_PARAMETER_UPDATE 12			/* Set 1/0 to enable/disable Camera Updating at ToDo_MainLoop() of your prog. */

/* MOUSE STATES */
#define MS_DLB 0x00
#define MS_LB 0x01
#define MS_DRB 0x02
#define MS_RB 0x03
#define MS_DMB 0x04
#define MS_MB 0x05
#define MS_WHEEL 0x06
#define MS_WHEEL_UP 120
#define MS_WHEEL_DOWN -120

/* KEYCODES */
#define VK_A 0x41	/* Letters A-Z */
#define VK_B 0x42
#define VK_C 0x43
#define VK_D 0x44
#define VK_E 0x45
#define VK_F 0x46
#define VK_G 0x47
#define VK_H 0x48
#define VK_I 0x49
#define VK_J 0x4A
#define VK_K 0x4B
#define VK_L 0x4C
#define VK_M 0x4D
#define VK_N 0x4E
#define VK_O 0x4F
#define VK_P 0x50
#define VK_Q 0x51
#define VK_R 0x52
#define VK_S 0x53
#define VK_T 0x54
#define VK_U 0x55
#define VK_V 0x56
#define VK_W 0x57
#define VK_X 0x58
#define VK_Y 0x59
#define VK_Z 0x5A

#define VK_0 0x30	/* Numbers 0-9 By Basic Keyboard */
#define VK_1 0x31
#define VK_2 0x32
#define VK_3 0x33
#define VK_4 0x34
#define VK_5 0x35
#define VK_6 0x36
#define VK_7 0x37
#define VK_8 0x38
#define VK_9 0x39

#define VK_N0 0x60 /* Numbers 0-9 By Num-Pad Keys */
#define VK_N1 0x61
#define VK_N2 0x62
#define VK_N3 0x63
#define VK_N4 0x64
#define VK_N5 0x65
#define VK_N6 0x66
#define VK_N7 0x67
#define VK_N8 0x68
#define VK_N9 0x69
/* For More References About Keys Check "lcc\include\win.h" at line ~ 6774 (VK_ESCAPE F.E.) */

#define BPLCC_GL_FONT_MODE_EXP2				0
#define BPLCC_GL_FONT_MODE_EXP				1
#define BPLCC_GL_FONT_MODE_LINEAR 		2
#define BPLCC_GL_FONT_HINT_DONTCARE		0
#define BPLCC_GL_FONT_HINT_NICEST			1
#define BPLCC_GL_FONT_HINT_FASTEST		2




#endif
